@@ -24,6 +24,7 @@ from typing import Optional
 
 import ewok
 import invoke
+import keyring
 import tabulate
 import tomlkit  # has more features than tomllib
 import yaml
@@ -435,11 +436,28 @@ def process_env_file(env_path: Path) -> dict[str, str]:
     return dict(values)
 
 
+def exists_nonempty(path: Path) -> bool:
+    """
+    Checks whether a given file path exists and is non-empty.
+
+    This function determines if the specified path represents an existing
+    file and whether it contains any data (i.e., its size is greater than 0).
+
+    Args:
+        path (Path): A Path object representing the file path to check.
+
+    Returns:
+        bool: True if the file exists and is non-empty, otherwise False.
+    """
+    return path.exists() and path.stat().st_size > 0
+
+
 def read_dotenv(env_path: Path = DEFAULT_DOTENV_PATH) -> dict[str, str]:
     """
     Read .env file from env_path and return a dict of key/value pairs.
 
-    If env_path is not given, TomlConfig.load().dotenv_path is used.
+    If the .env file doesn't exist at env_path, traverse up the directory tree
+    looking for one, stopping when a docker-compose.* file is found (project root).
 
     :param env_path: optional path to .env file
     :return: dict of key/value pairs from the .env file
@@ -453,8 +471,34 @@ def read_dotenv(env_path: Path = DEFAULT_DOTENV_PATH) -> dict[str, str]:
         # 'cache'
         return existing
 
-    items = process_env_file(env_path)
+    # First try the exact path provided
+    if exists_nonempty(env_path):
+        items = process_env_file(env_path)
+        _dotenv_settings[cache_key] = items
+        return items
 
+    # If not found and it's the default name, traverse up the tree
+    if env_path.name == DEFAULT_DOTENV_PATH.name:
+        current_dir = Path.cwd()
+
+        while current_dir != current_dir.parent:  # Stop at filesystem root
+            # Look for .env in current directory
+            potential_env = current_dir / DEFAULT_DOTENV_PATH.name
+            if exists_nonempty(potential_env):
+                items = process_env_file(potential_env)
+                _dotenv_settings[cache_key] = items
+                return items
+
+            # Check if we've reached a project root (docker-compose file exists)
+            if any(current_dir.glob("docker-compose.*")):
+                # Found project root, stop searching if we're not in the original directory
+                if current_dir != Path.cwd():
+                    break
+            # Move up one directory
+            current_dir = current_dir.parent
+
+    # If still not found, return empty dict (existing behavior)
+    items = {}
     _dotenv_settings[cache_key] = items
     return items
 
@@ -858,6 +902,22 @@ def load_dockercompose_with_includes(
         return {}
 
 
+def prompt_validate_sudo_pass(c: Context):
+    sudo_pass = getpass("Please enter the sudo password: ")
+    c.config.sudo.password = sudo_pass
+
+    try:
+        result = c.sudo("echo ''", warn=True, hide=True)
+        if not (result and result.ok):
+            raise invoke.exceptions.AuthFailure(result, "sudo")
+
+        cprint("Sudo password accepted!", color="green", file=sys.stderr)
+        return sudo_pass
+    except invoke.exceptions.AuthFailure as e:
+        cprint(str(e), color="red", file=sys.stderr)
+        return None
+
+
 @task()
 def require_sudo(c: Context) -> bool:
     """
@@ -879,18 +939,14 @@ def require_sudo(c: Context) -> bool:
         # prima
         return True
 
-    sudo_pass = getpass("Please enter the sudo password: ")
-    c.config.sudo.password = sudo_pass
+    with contextlib.suppress(Exception):
+        if current := keyring.get_password("edwh", "sudo"):
+            c.config.sudo.password = current
+            return True
 
-    try:
-        result = c.sudo("echo ''", warn=True, hide=True)
-        if not (result and result.ok):
-            raise invoke.exceptions.AuthFailure(result, "sudo")
-
-        cprint("Sudo password accepted!", color="green", file=sys.stderr)
+    if prompt_validate_sudo_pass(c):
         return True
-    except invoke.exceptions.AuthFailure as e:
-        cprint(str(e), color="red", file=sys.stderr)
+    else:
         cprint("Stopping now.")
         exit(1)
 
@@ -906,6 +962,37 @@ def build_toml(c: Context, overwrite: bool = False) -> TomlConfig | None:
     return write_user_input_to_config_toml(list(services.keys()), overwrite=overwrite)
 
 
+@task()
+def sudo(c: Context):
+    # 1.
+    # check current status in keyring
+    try:
+        current = keyring.get_password("edwh", "sudo")
+    except Exception:
+        current = None
+
+    # 2. change text based on current status (re-authorize)
+    if current:
+        allow = confirm(
+            "Would you like to re-authorize edwh to run sudo commands without password entry? [Yn]", default=True
+        )
+    else:
+        allow = confirm(
+            "Would you like to authorize edwh to run sudo commands without password entry? [yN]", default=False
+        )
+
+    if allow:
+        # if yes: add to keyring
+        if sudo_pass := prompt_validate_sudo_pass(c):
+            keyring.set_password("edwh", "sudo", sudo_pass)
+        else:
+            exit(1)
+
+    else:
+        # else: remove from keyring
+        keyring.delete_password("edwh", "sudo")
+
+
 @task(
     pre=[require_sudo],
     help={
@@ -1195,6 +1282,27 @@ def inspect_health(ctx, container: str, quiet: bool = False) -> dict:
     return result
 
 
+@task(
+    iterable=("service",),
+)
+def wait_until_healthy(ctx: Context, services: list[str] = (), quiet: bool = False):
+    initial_length = 0
+    # for every container with a health check, wait for it to be either healthy or dead (not starting)
+    while missing := [_.container for _ in get_healths(ctx, *services) if _ and _.health == "starting"]:
+        if not quiet:
+            msg = f" Waiting for {missing}" + " " * 25
+            if not initial_length:
+                initial_length = len(msg)
+
+            print(msg, end="\r")
+
+    # wait is done, now print empty line to cleanup print traces:
+    if initial_length and not quiet:
+        print(" " * initial_length)
+
+    return 0
+
+
 @task(
     flags={
         "show_all": ("all", "a"),
@@ -1237,25 +1345,11 @@ def health(
     else:
         services = []
 
-    healths = get_healths(ctx, *services)
-
     if wait:
-        initial_length = 0
-        # for every container with a health check, wait for it to be either healthy or dead (not starting)
-        while tracked := [_.container for _ in healths if _ and _.health == "starting"]:
-            if not quiet:
-                msg = f" Waiting for {tracked}" + " " * 25
-                if not initial_length:
-                    initial_length = len(msg)
+        return wait_until_healthy(ctx, services, quiet=quiet)
 
-                print(msg, end="\r")
-            healths = get_healths(ctx, *tracked)
-
-        # wait is done, now print empty line to cleanup print traces:
-        if initial_length and not quiet:
-            print(" " * initial_length)
-
-    elif not quiet:
+    healths = get_healths(ctx, *services)
+    if not quiet:
         for health_status in sorted((_ for _ in healths if _ is not None), key=lambda h: (h.level, h.container)):
             print(f"- {health_status}")
             if verbose and not health_status.ok and health_status.container_id:
@@ -1639,14 +1733,17 @@ def logs(
             return []
 
 
-def start_logs(c: Context, service: typing.Collection[str] = None):
+def start_logs(c: Context, service: typing.Collection[str] = None, args: str = ""):
     """
     Normal edwh logs can't just be called with `edwh.tasks.logs` so this wrapper makes it easier.
 
     (otherwise `logs` will try to elevate permissions and get confused due to not being called directly)
     """
     service = service_names(service or ())
-    return c.run("edwh logs " + " ".join(f"-s {s}" for s in service), pty=True)
+
+    args = " ".join(f"-s {s}" for s in service) + f" {args}"
+
+    return c.run(f"edwh logs {args}", pty=True)
 
 
 @task(
@@ -2052,7 +2149,7 @@ def clean_postgres(ctx: Context, yes: bool = False) -> None:
             pg_data_volumes.extend([mount["Name"] for mount in info["Mounts"] if "Name" in mount])
 
     # stop, remove the postgres instances and remove anonymous volumes
-    stop_remove_containers(ctx, "pg-0", "pg-1", "pgpool", "pg-stats")
+    stop_remove_containers(ctx, *containers)
 
     # remove images after containers have been stopped and removed
     if pg_data_volumes:
